package period

import (
	"errors"
	"github.com/jotaen/klog/klog"
	"regexp"
	"strconv"
)

type Year struct {
	date klog.Date
}

type YearHash Hash

var yearPattern = regexp.MustCompile(`^\d{4}$`)

func NewYearFromDate(d klog.Date) Year {
	return Year{d}
}

func NewYearFromString(yyyy string) (Year, error) {
	if !yearPattern.MatchString(yyyy) {
		return Year{}, errors.New("INVALID_YEAR_PERIOD")
	}
	year, err := strconv.Atoi(yyyy)
	if err != nil {
		return Year{}, errors.New("INVALID_YEAR_PERIOD")
	}
	d, dErr := klog.NewDate(year, 1, 1)
	if dErr != nil {
		return Year{}, errors.New("INVALID_YEAR_PERIOD")
	}
	return Year{d}, nil
}

func (y Year) Period() Period {
	since, _ := klog.NewDate(y.date.Year(), 1, 1)
	until, _ := klog.NewDate(y.date.Year(), 12, 31)
	return NewPeriod(since, until)
}

func (y Year) Previous() Year {
	lastYear, err := klog.NewDate(y.date.Year()-1, 1, 1)
	if err != nil {
		panic("Invalid year")
	}
	return Year{lastYear}
}

func (y Year) Hash() YearHash {
	hash := newBitMask()
	hash.populate(uint32(y.date.Year()), 10000)
	return YearHash(hash.Value())
}
